const fs = require("fs");
const Module = require("./module");

const UNKNOWN_MESSAGE = {
    format: "standard",
    message: "Unkown command"
}

class ModuleManager {
    constructor() {
        console.log("Creating module manager");
        this.moduleList = {};
        this.scanModules();
    }

    scanModules() {
        console.log("Scanning for modules");
        fs.readdirSync(process.env.MODULES_DIR).forEach(file => {
            let path = process.env.MODULES_DIR + "/" + file + "/package.json";

            if(fs.existsSync(path)) {
                this.loadNodeModule(path);
            }
        });
    }

    loadNodeModule(path) {
        let rawData = fs.readFileSync(path);
        let header = JSON.parse(rawData);

        let modulePath = path.substring(0, path.lastIndexOf('/'));
        let entrypoint = modulePath + "/" + header.main;

        this.moduleList[header.prefix] = new Module(header.name, 
            header.description, 
            header.prefix, 
            "node",
            entrypoint,
            header.version);
    }

    execute(prefix, message) {
        let module = this.moduleList[prefix];

        if(module === undefined) {
            this.reply(message, UNKNOWN_MESSAGE);

            return;
        }

        let rawResult = module.run(message);
        let result = JSON.parse(rawResult.toString());

        if(result.status === "success") {
            switch(result.action) {
                case "reply":
                    this.reply(message, result);
                    break;
                case "send":
                    this.send(message, result);
                default:
                    break;
            }
        }
    }

    reply(origin, result) {
        let message = undefined;

        if(result.format === "standard") {
            message = result.message;
        }

        if(result.format === "embed") {
            message = {embed: result.profile}
        }

        origin.reply(message);
    }
}

module.exports = ModuleManager;
